/*****************************************************************************
**  File Name:          cfs.c
**  Generated with:     Solstice TMN Agent Toolkit 2.0
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
**      The current file contains the implementation of the methods required
**      for the cfs class.
**      This class is defined in the following file:
**           cfs.gdmo
**
**----------------------------------------------------------------------------
**
**
**      The cfs class is registered as 1.3.6.1.4.1.42.2.2.3.20.2.2.1
**
**      The cfs class contains the following attributes:
**            cfsId
**            cfsName
**            cfsAutoUpdate
**            objectClass
**            nameBinding
**            packages
**            allomorphs
**
**      The cfs class supports the following actions:
**            cfsUpdateSystem
**
**      The cfs class supports the following notifications:
**            attributeValueChange
**
**      This class supports the create operation
**      This class supports the delete operation
**
**
*****************************************************************************/


/* 
** NOTE : 
** To find the places where you need to implement your code, 
** search for the string 'ADD YOUR'.
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifdef WIN32
#include <xom.h>
#include <xmp.h>
#include <xmp_cmis.h>
#include <usr_caDefs.h>
#include <usr_caGo.h>
#include <usr_caDmi.h>
#include <usr_caXom.h>
#include <usr_caCtrl.h>
#include <usr_caTree.h>
#else /* WIN32 */
#include <cmip/xom.h>
#include <cmip/xmp.h>
#include <cmip/xmp_cmis.h>
#include <tmn_agent/usr_caDefs.h>
#include <tmn_agent/usr_caGo.h>
#include <tmn_agent/usr_caDmi.h>
#include <tmn_agent/usr_caXom.h>
#include <tmn_agent/usr_caCtrl.h>
#include <tmn_agent/usr_caTree.h>
#endif /* WIN32 */
#include "metadata.h"

#include "cfs.h"



/*
** Object class OID (encoded)
*/


static OM_descriptor cfs_objectClass = {
	MP_ATTRIBUTE_VALUE,
	OM_S_ENCODING_STRING,
	{ 15, "\x80\x0d\x2b\x06\x01\x04\x01\x2a\x02\x02\x03\x14\x02\x02\x01" }
} ; /* 1.3.6.1.4.1.42.2.2.3.20.2.2.1 */

/*
** cfs_init
*/

OM_return_code cfs_init(
	ca_context_t	*context,
	void		*caSupRef,
	void		**caCurRef
)
{
	/*
	** ADD YOUR CODE HERE:
	** 
	** Use this function for initializing the class. This function is
	** called at agent startup before any callbacks.
	** For example, you may add code to create default object
	** instances of the class.
	**
	** The following parameters are used:
	**	caSupRef:	caRef to the superior moi (input)
	**	caCurRef:	caRef to an moi created by the function (output)
	**
	** This interface is defined for convenience and parameters used
	** may be changed.
	*/

	return OM_SUCCESS ;

}

/*
** cfs_create
*/

OM_return_code cfs_create(
	ca_context_t	*context,
	void		*caSupRef,
	ca_binding_t	*binding,
	OM_descriptor	*localName,
	void		**caRef,
	void		**accessRef,
	OM_descriptor	*error
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t *newInstance = NULL;

	*caRef = NULL;
	*accessRef = NULL;

	/*
	** Trace
	*/
	if (context->trace & CA_L_CALLBACK) {
		printf("CALL CREATE METHOD FOR cfs CLASS\n");
		if (binding != NULL) {
			printf("\tBinding    : %s\n", (binding->id)->name);
			if (binding->superior != NULL)
				printf("\tSuperior   : %s\n", 
						(binding->superior)->name);
			else
				printf("\tSuperior   : root\n") ;
			printf("\tNamed With : %s\n", 
						((binding->namedBy)->id)->name);
		}
		else {
			printf("\tNo name binding specified\n");
		}
	}

	/*
	** Allocate a new instance.
	*/
	newInstance = (cfs_t *)calloc(1, sizeof(cfs_t)) ;
	if (newInstance == NULL) {
		printf("cfs_create: calloc failed\n") ;
		ret = OM_FUNCTION_DECLINED ;
	}

	/*
	** Initialise the new instance :
	**
	** (1) check and initialize the naming attribute value
	*/
	if (ret == OM_SUCCESS) {

		if (binding->namedBy == &cfs_cfsId__def) {
			ret = dec_CFSObject_ASN1module_CfsId(
				localName,
				&(newInstance->cfsId)
			);
		}
		else { /* Name binding is invalid */
			if (context->trace & CA_L_CALLBACK) {
				printf("cfs_create : invalid name binding");
			}
			ret = OM_FUNCTION_DECLINED;
		}
		if (ret == OM_ENCODING_INVALID) {
			/* The naming value has a wrong syntax */
			error->type = MP_ERROR_STATUS;
			error->syntax = OM_S_ENUMERATION;
			error->value.enumeration = MP_E_INVALID_ATTRIBUTE_VALUE;
			if (context->trace & CA_L_CALLBACK) {
				printf("cfs_create : naming attribute syntax is invalid");
			}
			ret = OM_SUCCESS;
		}
	}

	/*
	** Initialise the new instance :
	**
	** (2) set default values for all the fields.
	*/
	if (ret == OM_SUCCESS) {
		newInstance->packagesPresent = OM_FALSE ;
		newInstance->allomorphsPresent = OM_FALSE ;
		newInstance->nameBinding = binding;

		/*
		** ADD YOUR CODE HERE:
		** Add your code to initialize remaining attribute values.
		*/
	}

	/*
	** Add the new instance to the MIT.
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_addInstWithSupInTreeFast(
			context,
			&cfs__def,
			binding->namedBy->id,
			localName,
			NULL,
			caSupRef,
			newInstance,
			caRef
		) ;
		if (ret == OM_SUCCESS) {
			ret = ca_registerFuncForDel(
				context,
				*caRef,
				cfs_free_instance
			) ;
		}
		if (ret != OM_SUCCESS) {
			printf("cfs_create : failed to add instance in tree !\n") ;
		}
	}

	/*
	** ADD YOUR CODE HERE:
	** The new instance is now created and inserted in the MIT.
	** You can add code here to update the managed resources.
	** In case of error, you need to :
	**	- remove newInstance from the MIT
	**	- set ret to OM_FUNCTION_DECLINED
	*/
	if (ret == OM_SUCCESS) {



	}


	/*
	** Undo the create if an error has occurred
	*/
	if ((ret != OM_SUCCESS) || (error->type == MP_ERROR_STATUS)) {
		if (*caRef != NULL) {
			ret = ca_delFinalNodeInTree(
				context,
				NULL,
				NULL,
				NULL,
				*caRef
			) ;
			/*
			**  Note that cfs_free_instance() 
			** is called automatically
			*/
			newInstance = NULL ;
		}
		if (newInstance != NULL) {
			ret = cfs_free_instance(
				context, 
				*caRef,
				newInstance
			) ;
		}
	}


	/*
	** Trace
	*/
	if (context->trace & CA_L_PRINT_TREE) {
		ca_examinTree(context) ;
	}
	if (context->trace & CA_L_CALLBACK) {
		printf("END OF CREATE METHOD FOR cfs CLASS\n") ;
	}

	return ret ;
}

/*
** cfs_delete
*/

OM_return_code cfs_delete(
	ca_context_t	*context,
	OM_sint32	invokeId,
	OM_boolean	type,
	void		*usrRef,
	void		*caRef,
	void		*accessRef
)
{
	OM_return_code ret = OM_SUCCESS ;
	OM_descriptor error = OM_NULL_DESCRIPTOR ;
	cfs_t *instance ;
	ca_list_t *children ;
	OM_uint childrenNb ;


	instance = (cfs_t *) usrRef ;

	/*
	** Trace
	*/
	if (context->trace & CA_L_CALLBACK) {
		printf("CALL DELETE METHOD FOR cfs CLASS\n") ;
	}

	/*
	** First check the deletion is authorized by the name binding.
	*/
	if ((instance->nameBinding->oper & CA_N_DELETE) == 0) {
		ret = OM_FUNCTION_DECLINED;
		if (context->trace & CA_L_CALLBACK) {
			printf("cfs_delete: deletion not supported\n") ;
		}
	}
	else if (instance->nameBinding->oper & CA_N_DELETE_NO) {
		ret = ca_findChildOfInst(context, NULL, NULL, usrRef, NULL,
						&children, &childrenNb) ;
		if (ret == OM_SUCCESS) {
			ca_freeListOfInst(children);
			if (childrenNb >= 1) {
				ret = OM_FUNCTION_DECLINED;
				if (context->trace & CA_L_CALLBACK) {
					printf("cfs_delete: deletion not supported\n") ;
				}
			}
		}
	}

	/*
	** ADD YOUR CODE HERE:
	** The instance is going to be deleted soon.
	** You can add code here to update the managed resources.
	** If an error occurs, set ret to OM_FUNCTION_DECLINED.
	*/
	if (ret == OM_SUCCESS) {


	}

	/*
	** Remove the instance from the tree.
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_delAllSubNodesInTree(context, NULL, NULL, NULL, caRef);
		if (ret == OM_SUCCESS) 
			ret = ca_delFinalNodeInTree(context, NULL, NULL, NULL, caRef);
		if (ret != OM_SUCCESS) {
			printf("cfs_delete : failed to remove instance !\n");
		}
		/*
		** Note that cfs_free_instance() 
		** is called automatically
		*/
	}

	/*
	** Call ca_gom_delete_rsp().
	*/
	if (type == OM_TRUE) {
		if (ret != OM_SUCCESS) {
			error.type = MP_ERROR_STATUS;
			error.syntax = OM_S_ENUMERATION;
			error.value.enumeration = MP_E_PROCESSING_FAILURE;
		}
		ret = ca_gom_delete_rsp(
			context, 
			invokeId, 
			usrRef, 
			accessRef, 
			&error
		);
	}

	/*
	** Trace
	*/
	if (context->trace & CA_L_PRINT_TREE) {
		ca_examinTree(context) ;
	}
	if (context->trace & CA_L_CALLBACK) {
		printf("END OF DELETE METHOD FOR cfs CLASS\n") ;
	}

	return ret ;
}


/*
** cfs_free_instance
*/

OM_return_code cfs_free_instance(
	ca_context_t	*context,
	void		*caRef,
	void		*usrRef
)
{
	cfs_t *instance = (cfs_t *)usrRef;

	free_CFSObject_ASN1module_CfsId(&(instance->cfsId));
	free_CFSObject_ASN1module_CfsDirectory(&(instance->cfsName));
	free_CFSObject_ASN1module_AutoUpdate(&(instance->cfsAutoUpdate));
	free_Attribute_ASN1Module_Packages(&(instance->packages));
	free_Attribute_ASN1Module_Allomorphs(&(instance->allomorphs));
	if (instance->attrValChngLst != NULL)
		ca_dmi_freeAttrChngLst(context, instance->attrValChngLst) ;

	/*
	** ADD YOUR CODE HERE:
	** If you have added some fields to the instance structure, you
	** can add code here to free the memory allocated to these fields.
	*/



	free(instance) ;

	return OM_SUCCESS ;
}


/*
** cfs_check
*/

OM_return_code cfs_check(
	ca_context_t	*context,
	OM_sint		oper,
	void 		*caRef,
	void 		*usrRef
)
{
	OM_return_code	ret ;
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;


	/*
	** Notification management
	*/
	if (oper == MP_SET_IND) {
		/*
		** Emit attribute value change notification.
		*/
		if (instance->attrValChngLst != NULL) {
			ret= ca_dmi_attrValChngNotif(
				context,
				usrRef,
				CA_DMI_T_SOURCEINDICATOR_management,
				-1,
				instance->attrValChngLst,
				NULL
			) ;
			if (ret == OM_SUCCESS)
				instance->attrValChngLst = NULL ;
		}
	}

	/*
	** Clean remaining notifications.
	*/
	if (instance->attrValChngLst != NULL) {
		ca_dmi_freeAttrChngLst(context, instance->attrValChngLst) ;
		instance->attrValChngLst = NULL ;
	}

	return OM_SUCCESS ;
}


/*
** cfs_cfsId_get
*/

OM_return_code cfs_cfsId_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;
	ret = enc_CFSObject_ASN1module_CfsId(
		&(instance->cfsId),
		value
	);

	return ret ;
}


/*
** cfs_cfsId_set
*/

OM_return_code cfs_cfsId_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
)
{
	OM_return_code ret;
	cfs_t *instance;
	CFSObject_ASN1module_CfsId decodedValue;

	instance = (cfs_t *) usrRef ;

	/*
	** Decode the new value
	*/
	ret = dec_CFSObject_ASN1module_CfsId(valueForOperation, &decodedValue) ;
	if (ret == OM_SUCCESS) {
		ret = ca_copy_xom_value(resultingValue, valueForOperation) ;
		if (ret != OM_SUCCESS) {
			free_CFSObject_ASN1module_CfsId(&decodedValue) ;
		}
	}
	else if (ret == OM_ENCODING_INVALID) {
		resultingValue->type = MP_ERROR_STATUS;
		resultingValue->syntax = OM_S_ENUMERATION;
		resultingValue->value.enumeration =
					MP_E_INVALID_ATTRIBUTE_VALUE;
	}
	/*
	** Copy the decoded value in the managed object
	*/
	if (ret == OM_SUCCESS) {
		free_CFSObject_ASN1module_CfsId(&(instance->cfsId)) ;
		instance->cfsId = decodedValue ;
	}

	/*
	** Include the attribute in the attribute value change notification
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_dmi_bldAttrChngLst(
			context, 
			&(instance->attrValChngLst),
			&cfsId__id, 
			NULL, 
			valueForOperation
		) ;
	}

	/*
	** ADD YOUR CODE HERE:
	** Managed object is now updated.
	** You can code here to update the managed resource.
	*/
	if (ret == OM_SUCCESS) {


	}

	/*
	** When a CMIS error is reported, return code must be cleared.
	*/
	if (resultingValue->type == MP_ERROR_STATUS) {
		ret = OM_SUCCESS ;
	}

	return ret ;
}


/*
** cfs_cfsId_eval
*/

OM_return_code cfs_cfsId_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t *instance;
	CFSObject_ASN1module_CfsId decodedValue;

	instance = (cfs_t *) usrRef ;

	if (type == MP_PRESENT) {
		*result = OM_TRUE ;
		*error = OM_FALSE ;
	}
	else {
		/*
		** Decode value1
		*/
		ret = dec_CFSObject_ASN1module_CfsId(value1, &decodedValue) ;

		/*
		** Compare the decoded value to the value stored
		** in the instance structure.
		*/
		if (ret == OM_SUCCESS) {
			switch(type) {

			case MP_EQUALITY :
			case MP_GREATER_OR_EQUAL :
			case MP_LESS_OR_EQUAL :
			case MP_SUBSTRINGS :
				*result = cmp_CFSObject_ASN1module_CfsId(
					&decodedValue,
					&instance->cfsId,
					type
				) ;
				*error = OM_FALSE ;
				break ;

			default :
				*result = OM_FALSE ;
				*error = OM_TRUE ;
				break ;

			}
		}
		else if (ret == OM_ENCODING_INVALID) {
			*error = OM_TRUE ;
			ret = OM_SUCCESS ; 
		}

		/*
		** Cleaning
		*/
		if (ret == OM_SUCCESS)
			free_CFSObject_ASN1module_CfsId(&decodedValue) ;
	}

	return ret ;
}


/*
** cfs_cfsName_get
*/

OM_return_code cfs_cfsName_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;
	ret = enc_CFSObject_ASN1module_CfsDirectory(
		&(instance->cfsName),
		value
	);

	return ret ;
}


/*
** cfs_cfsName_set
*/

OM_return_code cfs_cfsName_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
)
{
	OM_return_code ret;
	cfs_t *instance;
	CFSObject_ASN1module_CfsDirectory decodedValue;

	instance = (cfs_t *) usrRef ;

	/*
	** Decode the new value
	*/
	ret = dec_CFSObject_ASN1module_CfsDirectory(valueForOperation, &decodedValue) ;
	if (ret == OM_SUCCESS) {
		ret = ca_copy_xom_value(resultingValue, valueForOperation) ;
		if (ret != OM_SUCCESS) {
			free_CFSObject_ASN1module_CfsDirectory(&decodedValue) ;
		}
	}
	else if (ret == OM_ENCODING_INVALID) {
		resultingValue->type = MP_ERROR_STATUS;
		resultingValue->syntax = OM_S_ENUMERATION;
		resultingValue->value.enumeration =
					MP_E_INVALID_ATTRIBUTE_VALUE;
	}
	/*
	** Copy the decoded value in the managed object
	*/
	if (ret == OM_SUCCESS) {
		free_CFSObject_ASN1module_CfsDirectory(&(instance->cfsName)) ;
		instance->cfsName = decodedValue ;
	}

	/*
	** Include the attribute in the attribute value change notification
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_dmi_bldAttrChngLst(
			context, 
			&(instance->attrValChngLst),
			&cfsName__id, 
			NULL, 
			valueForOperation
		) ;
	}

	/*
	** ADD YOUR CODE HERE:
	** Managed object is now updated.
	** You can code here to update the managed resource.
	*/
	if (ret == OM_SUCCESS) {


	}

	/*
	** When a CMIS error is reported, return code must be cleared.
	*/
	if (resultingValue->type == MP_ERROR_STATUS) {
		ret = OM_SUCCESS ;
	}

	return ret ;
}


/*
** cfs_cfsName_eval
*/

OM_return_code cfs_cfsName_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t *instance;
	CFSObject_ASN1module_CfsDirectory decodedValue;

	instance = (cfs_t *) usrRef ;

	if (type == MP_PRESENT) {
		*result = OM_TRUE ;
		*error = OM_FALSE ;
	}
	else {
		/*
		** Decode value1
		*/
		ret = dec_CFSObject_ASN1module_CfsDirectory(value1, &decodedValue) ;

		/*
		** Compare the decoded value to the value stored
		** in the instance structure.
		*/
		if (ret == OM_SUCCESS) {
			switch(type) {

			case MP_EQUALITY :
			case MP_GREATER_OR_EQUAL :
			case MP_LESS_OR_EQUAL :
			case MP_SUBSTRINGS :
				*result = cmp_CFSObject_ASN1module_CfsDirectory(
					&decodedValue,
					&instance->cfsName,
					type
				) ;
				*error = OM_FALSE ;
				break ;

			default :
				*result = OM_FALSE ;
				*error = OM_TRUE ;
				break ;

			}
		}
		else if (ret == OM_ENCODING_INVALID) {
			*error = OM_TRUE ;
			ret = OM_SUCCESS ; 
		}

		/*
		** Cleaning
		*/
		if (ret == OM_SUCCESS)
			free_CFSObject_ASN1module_CfsDirectory(&decodedValue) ;
	}

	return ret ;
}


/*
** cfs_cfsAutoUpdate_get
*/

OM_return_code cfs_cfsAutoUpdate_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;
	ret = enc_CFSObject_ASN1module_AutoUpdate(
		&(instance->cfsAutoUpdate),
		value
	);

	return ret ;
}


/*
** cfs_cfsAutoUpdate_set
*/

OM_return_code cfs_cfsAutoUpdate_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
)
{
	OM_return_code ret;
	cfs_t *instance;
	CFSObject_ASN1module_AutoUpdate decodedValue;

	instance = (cfs_t *) usrRef ;

	/*
	** Decode the new value
	*/
	ret = dec_CFSObject_ASN1module_AutoUpdate(valueForOperation, &decodedValue) ;
	if (ret == OM_SUCCESS) {
		ret = ca_copy_xom_value(resultingValue, valueForOperation) ;
		if (ret != OM_SUCCESS) {
			free_CFSObject_ASN1module_AutoUpdate(&decodedValue) ;
		}
	}
	else if (ret == OM_ENCODING_INVALID) {
		resultingValue->type = MP_ERROR_STATUS;
		resultingValue->syntax = OM_S_ENUMERATION;
		resultingValue->value.enumeration =
					MP_E_INVALID_ATTRIBUTE_VALUE;
	}
	/*
	** Copy the decoded value in the managed object
	*/
	if (ret == OM_SUCCESS) {
		free_CFSObject_ASN1module_AutoUpdate(&(instance->cfsAutoUpdate)) ;
		instance->cfsAutoUpdate = decodedValue ;
	}

	/*
	** Include the attribute in the attribute value change notification
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_dmi_bldAttrChngLst(
			context, 
			&(instance->attrValChngLst),
			&cfsAutoUpdate__id, 
			NULL, 
			valueForOperation
		) ;
	}

	/*
	** ADD YOUR CODE HERE:
	** Managed object is now updated.
	** You can code here to update the managed resource.
	*/
	if (ret == OM_SUCCESS) {


	}

	/*
	** When a CMIS error is reported, return code must be cleared.
	*/
	if (resultingValue->type == MP_ERROR_STATUS) {
		ret = OM_SUCCESS ;
	}

	return ret ;
}


/*
** cfs_cfsAutoUpdate_eval
*/

OM_return_code cfs_cfsAutoUpdate_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t *instance;
	CFSObject_ASN1module_AutoUpdate decodedValue;

	instance = (cfs_t *) usrRef ;

	if (type == MP_PRESENT) {
		*result = OM_TRUE ;
		*error = OM_FALSE ;
	}
	else {
		/*
		** Decode value1
		*/
		ret = dec_CFSObject_ASN1module_AutoUpdate(value1, &decodedValue) ;

		/*
		** Compare the decoded value to the value stored
		** in the instance structure.
		*/
		if (ret == OM_SUCCESS) {
			switch(type) {

			case MP_EQUALITY :
				*result = cmp_CFSObject_ASN1module_AutoUpdate(
					&decodedValue,
					&instance->cfsAutoUpdate,
					type
				) ;
				*error = OM_FALSE ;
				break ;

			default :
				*result = OM_FALSE ;
				*error = OM_TRUE ;
				break ;

			}
		}
		else if (ret == OM_ENCODING_INVALID) {
			*error = OM_TRUE ;
			ret = OM_SUCCESS ; 
		}

		/*
		** Cleaning
		*/
		if (ret == OM_SUCCESS)
			free_CFSObject_ASN1module_AutoUpdate(&decodedValue) ;
	}

	return ret ;
}


/*
** cfs_objectClass_get
*/

OM_return_code cfs_objectClass_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	value->syntax = cfs_objectClass.syntax ;
	value->value  = cfs_objectClass.value ;

	return OM_SUCCESS;
}


/*
** cfs_nameBinding_get
*/

OM_return_code cfs_nameBinding_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;
	if (instance->nameBinding != NULL) {
		value->syntax = instance->nameBinding->id->priv.syntax ;
		value->value = instance->nameBinding->id->priv.value ;
	}
	else {
		value->syntax = OM_S_NULL ;
	}

	return OM_SUCCESS;
}


/*
** cfs_packages_get
*/

OM_return_code cfs_packages_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;
	if (instance->packagesPresent == OM_TRUE) {
		ret = enc_Attribute_ASN1Module_Packages(
			&(instance->packages), 
			value
		) ;
	}
	else {
		value->type = MP_ERROR_STATUS;
		value->syntax = OM_S_ENUMERATION;
		value->value.enumeration=MP_E_NO_SUCH_ATTRIBUTE;
		ret = OM_SUCCESS ;
	}
	return ret ;
}


/*
** cfs_packages_set
*/

OM_return_code cfs_packages_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
)
{
	OM_return_code ret;
	cfs_t *instance;
	Attribute_ASN1Module_Packages decodedValue;

	instance = (cfs_t *) usrRef ;

	/*
	** ADD YOUR CODE HERE:
	** The attribute is conditional.
	** By default the setting of the attribute is allowed.
	** You can add your code below to disallow setting of
	** the attribute for the current instance.
	*/

	/*
	** Decode the new value
	*/
	ret = dec_Attribute_ASN1Module_Packages(valueForOperation, &decodedValue) ;
	if (ret == OM_SUCCESS) {
		ret = ca_copy_xom_value(resultingValue, valueForOperation) ;
		if (ret != OM_SUCCESS) {
			free_Attribute_ASN1Module_Packages(&decodedValue) ;
		}
	}
	else if (ret == OM_ENCODING_INVALID) {
		resultingValue->type = MP_ERROR_STATUS;
		resultingValue->syntax = OM_S_ENUMERATION;
		resultingValue->value.enumeration =
					MP_E_INVALID_ATTRIBUTE_VALUE;
	}
	/*
	** Copy the decoded value in the managed object
	*/
	if (ret == OM_SUCCESS) {
		if (instance->packagesPresent)
			free_Attribute_ASN1Module_Packages(&(instance->packages)) ;
		instance->packages = decodedValue ;
		instance->packagesPresent = OM_TRUE ;
	}

	/*
	** Include the attribute in the attribute value change notification
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_dmi_bldAttrChngLst(
			context, 
			&(instance->attrValChngLst),
			&packages__id, 
			NULL, 
			valueForOperation
		) ;
	}

	/*
	** ADD YOUR CODE HERE:
	** Managed object is now updated.
	** You can code here to update the managed resource.
	*/
	if (ret == OM_SUCCESS) {


	}

	/*
	** When a CMIS error is reported, return code must be cleared.
	*/
	if (resultingValue->type == MP_ERROR_STATUS) {
		ret = OM_SUCCESS ;
	}

	return ret ;
}


/*
** cfs_packages_eval
*/

OM_return_code cfs_packages_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t *instance;
	Attribute_ASN1Module_Packages decodedValue;

	instance = (cfs_t *) usrRef ;

	if (type == MP_PRESENT) {
		*result = instance->packagesPresent ;
		*error = OM_FALSE ;
	}
	else {
		/*
		** Decode value1
		*/
		ret = dec_Attribute_ASN1Module_Packages(value1, &decodedValue) ;

		/*
		** Compare the decoded value to the value stored
		** in the instance structure.
		*/
		if (ret == OM_SUCCESS) {
			switch(type) {

			case MP_EQUALITY :
			case MP_SUBSET_OF :
			case MP_SUPERSET_OF :
			case MP_NON_NULL_SET_INTERSECTION :
				*result = cmp_Attribute_ASN1Module_Packages(
					&decodedValue,
					&instance->packages,
					type
				) ;
				*error = OM_FALSE ;
				break ;

			default :
				*result = OM_FALSE ;
				*error = OM_TRUE ;
				break ;

			}
		}
		else if (ret == OM_ENCODING_INVALID) {
			*error = OM_TRUE ;
			ret = OM_SUCCESS ; 
		}

		/*
		** Cleaning
		*/
		if (ret == OM_SUCCESS)
			free_Attribute_ASN1Module_Packages(&decodedValue) ;
	}

	return ret ;
}


/*
** cfs_allomorphs_get
*/

OM_return_code cfs_allomorphs_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;
	if (instance->allomorphsPresent == OM_TRUE) {
		ret = enc_Attribute_ASN1Module_Allomorphs(
			&(instance->allomorphs), 
			value
		) ;
	}
	else {
		value->type = MP_ERROR_STATUS;
		value->syntax = OM_S_ENUMERATION;
		value->value.enumeration=MP_E_NO_SUCH_ATTRIBUTE;
		ret = OM_SUCCESS ;
	}
	return ret ;
}


/*
** cfs_allomorphs_set
*/

OM_return_code cfs_allomorphs_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
)
{
	OM_return_code ret;
	cfs_t *instance;
	Attribute_ASN1Module_Allomorphs decodedValue;

	instance = (cfs_t *) usrRef ;

	/*
	** ADD YOUR CODE HERE:
	** The attribute is conditional.
	** By default the setting of the attribute is allowed.
	** You can add your code below to disallow setting of
	** the attribute for the current instance.
	*/

	/*
	** Decode the new value
	*/
	ret = dec_Attribute_ASN1Module_Allomorphs(valueForOperation, &decodedValue) ;
	if (ret == OM_SUCCESS) {
		ret = ca_copy_xom_value(resultingValue, valueForOperation) ;
		if (ret != OM_SUCCESS) {
			free_Attribute_ASN1Module_Allomorphs(&decodedValue) ;
		}
	}
	else if (ret == OM_ENCODING_INVALID) {
		resultingValue->type = MP_ERROR_STATUS;
		resultingValue->syntax = OM_S_ENUMERATION;
		resultingValue->value.enumeration =
					MP_E_INVALID_ATTRIBUTE_VALUE;
	}
	/*
	** Copy the decoded value in the managed object
	*/
	if (ret == OM_SUCCESS) {
		if (instance->allomorphsPresent)
			free_Attribute_ASN1Module_Allomorphs(&(instance->allomorphs)) ;
		instance->allomorphs = decodedValue ;
		instance->allomorphsPresent = OM_TRUE ;
	}

	/*
	** Include the attribute in the attribute value change notification
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_dmi_bldAttrChngLst(
			context, 
			&(instance->attrValChngLst),
			&allomorphs__id, 
			NULL, 
			valueForOperation
		) ;
	}

	/*
	** ADD YOUR CODE HERE:
	** Managed object is now updated.
	** You can code here to update the managed resource.
	*/
	if (ret == OM_SUCCESS) {


	}

	/*
	** When a CMIS error is reported, return code must be cleared.
	*/
	if (resultingValue->type == MP_ERROR_STATUS) {
		ret = OM_SUCCESS ;
	}

	return ret ;
}


/*
** cfs_allomorphs_eval
*/

OM_return_code cfs_allomorphs_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t *instance;
	Attribute_ASN1Module_Allomorphs decodedValue;

	instance = (cfs_t *) usrRef ;

	if (type == MP_PRESENT) {
		*result = instance->allomorphsPresent ;
		*error = OM_FALSE ;
	}
	else {
		/*
		** Decode value1
		*/
		ret = dec_Attribute_ASN1Module_Allomorphs(value1, &decodedValue) ;

		/*
		** Compare the decoded value to the value stored
		** in the instance structure.
		*/
		if (ret == OM_SUCCESS) {
			switch(type) {

			case MP_EQUALITY :
			case MP_SUBSET_OF :
			case MP_SUPERSET_OF :
			case MP_NON_NULL_SET_INTERSECTION :
				*result = cmp_Attribute_ASN1Module_Allomorphs(
					&decodedValue,
					&instance->allomorphs,
					type
				) ;
				*error = OM_FALSE ;
				break ;

			default :
				*result = OM_FALSE ;
				*error = OM_TRUE ;
				break ;

			}
		}
		else if (ret == OM_ENCODING_INVALID) {
			*error = OM_TRUE ;
			ret = OM_SUCCESS ; 
		}

		/*
		** Cleaning
		*/
		if (ret == OM_SUCCESS)
			free_Attribute_ASN1Module_Allomorphs(&decodedValue) ;
	}

	return ret ;
}


/*
** cfs_cfsUpdateSystem
*/

OM_return_code cfs_cfsUpdateSystem(
	ca_context_t	*context,
	OM_sint32	invokeId,
	OM_uint		mode,
	OM_descriptor	*infoArg,
	void		*usrRef,
	void		*accessRef
)
{
	OM_return_code ret = OM_SUCCESS;
	OM_descriptor replyArg = OM_NULL_DESCRIPTOR;
	cfs_t * instance ;

	instance = (cfs_t *) usrRef ;

	/*
	** ADD YOUR CODE HERE:
	** Implement the action here.
	*/


	/*
	** Call ca_gom_action_rsp()
	*/
	if ((ret == OM_SUCCESS) || (replyArg.type == MP_ERROR_STATUS)){
		if (mode == MP_T_CONFIRMED) {
			ret = ca_gom_action_rsp(
				context, 
				invokeId, 
				usrRef, 
				accessRef, 
				&replyArg, 
				OM_FALSE
			);
		}
	}

	/*
	** When a CMIS error is reported, return code must be cleared.
	*/
	if (replyArg.type == MP_ERROR_STATUS) {
		ret = OM_SUCCESS ;
	}

	return ret;
}
