#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifdef WIN32
#include <xom.h>
#include <xmp.h>
#include <xmp_cmis.h>
#include <usr_caDefs.h>
#include <usr_caGo.h>
#include <usr_caDmi.h>
#include <usr_caXom.h>
#include <usr_caCtrl.h>
#include <usr_caTree.h>
#include <usr_caNoti.h>
#include <usr_caEfd.h>
#include <usr_caNs.h>
#include <usr_caLog.h>
#else /* WIN32 */
#include <cmip/xom.h>
#include <cmip/xmp.h>
#include <cmip/xmp_cmis.h>
#include <tmn_agent/usr_caDefs.h>
#include <tmn_agent/usr_caGo.h>
#include <tmn_agent/usr_caDmi.h>
#include <tmn_agent/usr_caXom.h>
#include <tmn_agent/usr_caCtrl.h>
#include <tmn_agent/usr_caTree.h>
#include <tmn_agent/usr_caNoti.h>
#include <tmn_agent/usr_caEfd.h>
#include <tmn_agent/usr_caNs.h>
#include <tmn_agent/usr_caLog.h>
#endif /* WIN32 */

#include "metadata.h"

#include "cfs.h"
#include "cfsFile.h"
#include "cfsDir.h"
#include "system.h"
#include "dmi_config.h"

#include "init.h"


/*
** Name of the default EFD
*/
OM_descriptor efdAva = {
	OM_NO_MORE_TYPES,
	OM_S_GRAPHIC_STRING,
	OM_STRING("efd")
} ;

/*
** AE-title of the default destination
*/
OM_descriptor defDestination = {
	OM_NO_MORE_TYPES,
	OM_S_OBJECT_IDENTIFIER_STRING,
OM_STRING("\x2b\x06\x01\x04\x01\x2a\x02\x02\x03\x02\x64")
} ;

/*
** Name of the default LOG
*/
OM_descriptor logAva = {
	OM_NO_MORE_TYPES,
	OM_S_GRAPHIC_STRING,
	OM_STRING("log")
} ;



/*
** general_init
*/

OM_return_code general_init(
	ca_context_t	*context
)
{

	OM_return_code ret ;
	void *caEfdRef = NULL ;
	void *caLogRef = NULL ;
	void *caSupRef = NULL ;
	void *caCurRef = NULL ;

	/*
	** Trace
	*/
	if (context->trace & CA_L_CALLBACK) {
		printf("CALL GENERAL INIT\n") ;
	}

	/*
	** call system_init
	*/
	ret = system_init(context, caSupRef, &caCurRef) ;

	if (ret != OM_SUCCESS) {
		printf("general_init: call to system_init failed\n") ;
		return ret ;
	}

	caSupRef = caCurRef ; 

	/*
	** Initialize DMI metadata
	*/
	ret = ca_dmi_object_init(context,  /* core agent context         */
	                listOfGenericAtt,  /* list of generic attributes */
	                listOfPseudoClass, /* list of notifications      */
	                NULL,	      /* oid of name binding        */
	                &system__def,       /* superior class to be used  */
	                CA_EFD_ALL_PACKAGES|CA_LOG_ALL_PACKAGES
	) ;

	if (ret != OM_SUCCESS) {
		printf("general_init: call to ca_dmi_object_init failed\n") ;
		return ret ;
	}

	/*
	** Initialize the Name Server
	*/
	ret = ns_init(context,        /* core agent context              */
	              caSupRef,       /* superior instance of ns objects */
	              "config.ns",  /* main translation file name      */
	              "config.ns"	 /* backup translation file name    */
	) ;

	if (ret != OM_SUCCESS) {
		printf("general_init: call to ns_init failed\n") ;
		return ret ;
	}

	/*
	** Create a default EFD
	*/
	ret = ca_dmi_efd_create(
		context,
		&efdAva,
		OM_FALSE,
		&defDestination,
		MP_ABSENT_OBJECT,
		&caEfdRef
	) ;
	if (ret != OM_SUCCESS) {
		printf("general_init: call to ca_dmi_efd_create failed\n") ;
		return ret ;
	}

	/*
	** Create a default LOG
	*/
	ret = ca_dmi_log_create(
		context,
		&logAva,
		OM_FALSE,
		64000, /* Default log size : 64kb */
		MP_ABSENT_OBJECT,
		&caLogRef
	) ;
	if (ret != OM_SUCCESS) {
		printf("general_init: call to ca_dmi_log_create failed\n") ;
		return ret ;
	}

	/*
	** call cfs_init
	*/
	ret = cfs_init(context, caSupRef, &caCurRef) ;

	if (ret != OM_SUCCESS) {
		printf("general_init: call to cfs_init failed\n") ;
		return ret ;
	}

	caSupRef = caCurRef ; 

	/*
	** call cfsFile_init
	*/
	ret = cfsFile_init(context, caSupRef, &caCurRef) ;

	if (ret != OM_SUCCESS) {
		printf("general_init: call to cfsFile_init failed\n") ;
		return ret ;
	}

	caSupRef = caCurRef ; 

	/*
	** call cfsDir_init
	*/
	ret = cfsDir_init(context, caSupRef, &caCurRef) ;

	if (ret != OM_SUCCESS) {
		printf("general_init: call to cfsDir_init failed\n") ;
		return ret ;
	}

	caSupRef = caCurRef ; 


	/*
	** Trace
	*/
	if (context->trace & CA_L_PRINT_TREE) {
		ca_examinTree(context) ;
	}
	if (context->trace & CA_L_CALLBACK) {
		printf("END OF GENERAL INIT\n") ;
	}

	return OM_SUCCESS ;

}
