#include "metadata.h"
#ifdef WIN32
#include <usr_caGo.h>
#include <usr_caNoti.h>
#include <usr_caNs.h>
#include <usr_caEfd.h>
#include <usr_caLog.h>
#else /* WIN32 */
#include <tmn_agent/usr_caGo.h>
#include <tmn_agent/usr_caNoti.h>
#include <tmn_agent/usr_caNs.h>
#include <tmn_agent/usr_caEfd.h>
#include <tmn_agent/usr_caLog.h>
#endif /* WIN32 */

#include "cfs.h"
#include "cfsFile.h"
#include "cfsDir.h"
#include "system.h"


/*
** List of classes
*/

ca_class_t * listOfClass[] = {
	&cfs__def,
	&cfsFile__def,
	&cfsDir__def,
	&system__def,
	efdClass,
	nsClass,
	nsEntryClass,
	logClass,
	&actualClass__def,
	NULL
} ;



/*
** cfs
*/

ca_instance_t cfs__instance = {
	cfs_create,
	cfs_delete,
	cfs_check
} ;

ca_attribute_t cfs_cfsId__def = {
	&cfsId__id,
	CA_O_GET,
	cfs_cfsId_get,
	cfs_cfsId_set,
	cfs_cfsId_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t cfs_cfsName__def = {
	&cfsName__id,
	CA_O_GET,
	cfs_cfsName_get,
	cfs_cfsName_set,
	cfs_cfsName_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t cfs_cfsAutoUpdate__def = {
	&cfsAutoUpdate__id,
	CA_O_GET | CA_O_REP,
	cfs_cfsAutoUpdate_get,
	cfs_cfsAutoUpdate_set,
	cfs_cfsAutoUpdate_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t cfs_objectClass__def = {
	&objectClass__id,
	CA_O_GET,
	cfs_objectClass_get,
	NULL,
	NULL,
	NULL
} ;

ca_attribute_t cfs_nameBinding__def = {
	&nameBinding__id,
	CA_O_GET,
	cfs_nameBinding_get,
	NULL,
	NULL,
	NULL
} ;

ca_attribute_t cfs_packages__def = {
	&packages__id,
	CA_O_GET,
	cfs_packages_get,
	cfs_packages_set,
	cfs_packages_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t cfs_allomorphs__def = {
	&allomorphs__id,
	CA_O_GET,
	cfs_allomorphs_get,
	cfs_allomorphs_set,
	cfs_allomorphs_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t * cfs__attr[] = {
	&cfs_cfsId__def,
	&cfs_cfsName__def,
	&cfs_cfsAutoUpdate__def,
	&cfs_objectClass__def,
	&cfs_nameBinding__def,
	&cfs_packages__def,
	&cfs_allomorphs__def,
	NULL
} ;

ca_action_t cfs__action[] = {
	{
		&cfsUpdateSystem__id,
		cfs_cfsUpdateSystem,
		ca_gomFreeEncodingMethod
	},
	{
		NULL,
		NULL,
		NULL
	}
} ;

ca_binding_t cfs__binding[] = {
	{
		&cfs_system__id,
		CA_N_NOTHING,
		&system__def,
		&cfs_cfsId__def
	},
	{
		NULL,
		CA_N_NOTHING,
		NULL,
		NULL
	}
} ;

ca_gdmo_t cfs_gdmo = {
	NULL,
	cfs__binding,
	cfs__attr,
	&cfs__instance,
	NULL,
	cfs__action
} ;


ca_class_t cfs__def = {
	"cfs",
	"1.3.6.1.4.1.42.2.2.3.20.2.2.1",
	&cfs_gdmo,
	{ 0, 0, {0}},
	ca_gomActMethod,
	NULL,
	NULL,
	ca_gomCreMethod,
	ca_gomDelMethod,
	ca_gomGetMethod,
	NULL,
	NULL,
	ca_gomSetMethod,
	CA_T_BASIC,
	CA_F_FILTER,
	NULL
} ;




/*
** cfsFile
*/

ca_instance_t cfsFile__instance = {
	cfsFile_create,
	cfsFile_delete,
	cfsFile_check
} ;

ca_attribute_t cfsFile_cfsId__def = {
	&cfsId__id,
	CA_O_GET,
	cfsFile_cfsId_get,
	cfsFile_cfsId_set,
	cfsFile_cfsId_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t cfsFile_cfsFilePermissions__def = {
	&cfsFilePermissions__id,
	CA_O_GET | CA_O_REP,
	cfsFile_cfsFilePermissions_get,
	cfsFile_cfsFilePermissions_set,
	cfsFile_cfsFilePermissions_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t cfsFile_objectClass__def = {
	&objectClass__id,
	CA_O_GET,
	cfsFile_objectClass_get,
	NULL,
	NULL,
	NULL
} ;

ca_attribute_t cfsFile_nameBinding__def = {
	&nameBinding__id,
	CA_O_GET,
	cfsFile_nameBinding_get,
	NULL,
	NULL,
	NULL
} ;

ca_attribute_t cfsFile_packages__def = {
	&packages__id,
	CA_O_GET,
	cfsFile_packages_get,
	cfsFile_packages_set,
	cfsFile_packages_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t cfsFile_allomorphs__def = {
	&allomorphs__id,
	CA_O_GET,
	cfsFile_allomorphs_get,
	cfsFile_allomorphs_set,
	cfsFile_allomorphs_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t * cfsFile__attr[] = {
	&cfsFile_cfsId__def,
	&cfsFile_cfsFilePermissions__def,
	&cfsFile_objectClass__def,
	&cfsFile_nameBinding__def,
	&cfsFile_packages__def,
	&cfsFile_allomorphs__def,
	NULL
} ;

ca_action_t cfsFile__action[] = {
	{
		&cfsUpdateFile__id,
		cfsFile_cfsUpdateFile,
		ca_gomFreeEncodingMethod
	},
	{
		NULL,
		NULL,
		NULL
	}
} ;

ca_binding_t cfsFile__binding[] = {
	{
		&cfsFile_cfsDir__id,
		CA_N_CREATE | CA_N_DELETE | CA_N_DELETE_ALL,
		&cfsDir__def,
		&cfsFile_cfsId__def
	},
	{
		&cfsFile_cfs__id,
		CA_N_CREATE | CA_N_DELETE | CA_N_DELETE_ALL,
		&cfs__def,
		&cfsFile_cfsId__def
	},
	{
		NULL,
		CA_N_NOTHING,
		NULL,
		NULL
	}
} ;

ca_gdmo_t cfsFile_gdmo = {
	NULL,
	cfsFile__binding,
	cfsFile__attr,
	&cfsFile__instance,
	NULL,
	cfsFile__action
} ;


ca_class_t cfsFile__def = {
	"cfsFile",
	"1.3.6.1.4.1.42.2.2.3.20.2.2.3",
	&cfsFile_gdmo,
	{ 0, 0, {0}},
	ca_gomActMethod,
	NULL,
	NULL,
	ca_gomCreMethod,
	ca_gomDelMethod,
	ca_gomGetMethod,
	NULL,
	NULL,
	ca_gomSetMethod,
	CA_T_BASIC,
	CA_F_FILTER,
	NULL
} ;




/*
** cfsDir
*/

ca_instance_t cfsDir__instance = {
	cfsDir_create,
	cfsDir_delete,
	cfsDir_check
} ;

ca_attribute_t cfsDir_cfsId__def = {
	&cfsId__id,
	CA_O_GET,
	cfsDir_cfsId_get,
	cfsDir_cfsId_set,
	cfsDir_cfsId_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t cfsDir_cfsFilePermissions__def = {
	&cfsFilePermissions__id,
	CA_O_GET | CA_O_REP,
	cfsDir_cfsFilePermissions_get,
	cfsDir_cfsFilePermissions_set,
	cfsDir_cfsFilePermissions_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t cfsDir_objectClass__def = {
	&objectClass__id,
	CA_O_GET,
	cfsDir_objectClass_get,
	NULL,
	NULL,
	NULL
} ;

ca_attribute_t cfsDir_nameBinding__def = {
	&nameBinding__id,
	CA_O_GET,
	cfsDir_nameBinding_get,
	NULL,
	NULL,
	NULL
} ;

ca_attribute_t cfsDir_packages__def = {
	&packages__id,
	CA_O_GET,
	cfsDir_packages_get,
	cfsDir_packages_set,
	cfsDir_packages_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t cfsDir_allomorphs__def = {
	&allomorphs__id,
	CA_O_GET,
	cfsDir_allomorphs_get,
	cfsDir_allomorphs_set,
	cfsDir_allomorphs_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t * cfsDir__attr[] = {
	&cfsDir_cfsId__def,
	&cfsDir_cfsFilePermissions__def,
	&cfsDir_objectClass__def,
	&cfsDir_nameBinding__def,
	&cfsDir_packages__def,
	&cfsDir_allomorphs__def,
	NULL
} ;

ca_action_t cfsDir__action[] = {
	{
		&cfsUpdateFile__id,
		cfsDir_cfsUpdateFile,
		ca_gomFreeEncodingMethod
	},
	{
		NULL,
		NULL,
		NULL
	}
} ;

ca_binding_t cfsDir__binding[] = {
	{
		&cfsFile_cfsDir__id,
		CA_N_CREATE | CA_N_DELETE | CA_N_DELETE_ALL,
		&cfsDir__def,
		&cfsDir_cfsId__def
	},
	{
		&cfsDir_cfs__id,
		CA_N_CREATE | CA_N_DELETE | CA_N_DELETE_ALL,
		&cfs__def,
		&cfsDir_cfsId__def
	},
	{
		NULL,
		CA_N_NOTHING,
		NULL,
		NULL
	}
} ;

ca_gdmo_t cfsDir_gdmo = {
	NULL,
	cfsDir__binding,
	cfsDir__attr,
	&cfsDir__instance,
	NULL,
	cfsDir__action
} ;


ca_class_t cfsDir__def = {
	"cfsDir",
	"1.3.6.1.4.1.42.2.2.3.20.2.2.2",
	&cfsDir_gdmo,
	{ 0, 0, {0}},
	ca_gomActMethod,
	NULL,
	NULL,
	ca_gomCreMethod,
	ca_gomDelMethod,
	ca_gomGetMethod,
	NULL,
	NULL,
	ca_gomSetMethod,
	CA_T_BASIC,
	CA_F_FILTER,
	NULL
} ;




/*
** system
*/

ca_instance_t system__instance = {
	NULL,
	NULL,
	system_check
} ;

ca_attribute_t system_systemId__def = {
	&systemId__id,
	CA_O_GET,
	system_systemId_get,
	NULL,
	system_systemId_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t system_systemTitle__def = {
	&systemTitle__id,
	CA_O_GET,
	system_systemTitle_get,
	NULL,
	system_systemTitle_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t system_operationalState__def = {
	&operationalState__id,
	CA_O_GET,
	system_operationalState_get,
	NULL,
	system_operationalState_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t system_usageState__def = {
	&usageState__id,
	CA_O_GET,
	system_usageState_get,
	NULL,
	system_usageState_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t system_objectClass__def = {
	&objectClass__id,
	CA_O_GET,
	system_objectClass_get,
	NULL,
	NULL,
	NULL
} ;

ca_attribute_t system_nameBinding__def = {
	&nameBinding__id,
	CA_O_GET,
	system_nameBinding_get,
	NULL,
	NULL,
	NULL
} ;

ca_attribute_t system_administrativeState__def = {
	&administrativeState__id,
	CA_O_GET | CA_O_REP,
	system_administrativeState_get,
	system_administrativeState_set,
	system_administrativeState_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t system_supportedFeatures__def = {
	&supportedFeatures__id,
	CA_O_ADD | CA_O_GET | CA_O_REM | CA_O_REP,
	system_supportedFeatures_get,
	system_supportedFeatures_set,
	system_supportedFeatures_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t system_packages__def = {
	&packages__id,
	CA_O_GET,
	system_packages_get,
	NULL,
	system_packages_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t system_allomorphs__def = {
	&allomorphs__id,
	CA_O_GET,
	system_allomorphs_get,
	NULL,
	system_allomorphs_eval,
	ca_gomFreeEncodingMethod
} ;

ca_attribute_t * system__attr[] = {
	&system_systemId__def,
	&system_systemTitle__def,
	&system_operationalState__def,
	&system_usageState__def,
	&system_objectClass__def,
	&system_nameBinding__def,
	&system_administrativeState__def,
	&system_supportedFeatures__def,
	&system_packages__def,
	&system_allomorphs__def,
	NULL
} ;

ca_gdmo_t system_gdmo = {
	NULL,
	NULL,
	system__attr,
	&system__instance,
	NULL,
	NULL
} ;


ca_class_t system__def = {
	"system",
	"2.9.3.2.3.13",
	&system_gdmo,
	{ 0, 0, {0}},
	ca_gomActMethod,
	NULL,
	NULL,
	ca_gomCreMethod,
	ca_gomDelMethod,
	ca_gomGetMethod,
	NULL,
	NULL,
	ca_gomSetMethod,
	CA_T_BASIC,
	CA_F_FILTER,
	NULL
} ;




/*
** actualClass
*/

ca_class_t actualClass__def = {
	"actualClass",
	"2.9.3.4.3.42",
	NULL,
	{ 0, 0, {0}},
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	CA_T_BASIC,
	CA_F_ACTUAL,
	NULL
} ;




